#
# License: BSD
#    https://raw.githubusercontent.com/stonier/groot_ansible/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Ansible subcommands for ros management.
"""

##############################################################################
# Imports
##############################################################################

import argparse
import platform
import subprocess

from . import common
from . import console

##############################################################################
# Methods
##############################################################################


def add_ros_arguments(parser):
    """
    Arguments relevant for a ros system.

    :param argparse.ArgumentParser parser:
    """
    group = parser.add_argument_group(title="ros arguments")
    group.add_argument('--rosdistro', action='store', default=None, help='manually specify the ros release to install/update')
#     group.add_argument('--only-upgrade', action='store_true', help='only upgrade currently installed ros debians')
#     group.add_argument('--only-rosdeps', action='store_true', help='only do the --install-rosdeps step')
#     group.add_argument('--skip-rosdeps', action='store_true', help='skip the --install-rosdeps step')


def guess_rosdistro():
    distro_map = {'trusty': 'indigo', 'xenial': 'kinetic'}
    unused_os, unused_version, codename = platform.linux_distribution()
    return distro_map[codename] if codename in distro_map.keys() else None  # let the role decide the default if we don't guess it


def parse_args(args):
    """
    Launches the playbooks for managing the ros software environment on a pc.
    """
    console.banner("'ros'")
    connection = "-c local"
    list_tasks = "--list-tasks" if args.list_tasks else ""
    # TODO check for a valid rosdistro on this platform
    rosdistro = args.rosdistro if args.rosdistro else guess_rosdistro()
    variable_ros_release = "-e ros_release={0}".format(rosdistro) if rosdistro else ""
    cmd = "ansible-playbook ros.yaml -K -i localhost, {connection} {list_tasks} {variable_ros_release}".format(**locals())
    cmd = common.append_verbosity_argument(cmd, args.verbose)
    console.key_value_pairs("Parameters", {"Rosdistro": rosdistro}, 10)
    console.key_value_pairs("Ansible", {"Command": cmd}, 10)
    print("")
    subprocess.call(cmd, cwd=args.home, shell=True)


def add_subparser(subparsers):
    """
    Add our own argparser to the parent.

    :param subparsers: the subparsers factory from the parent argparser.
    """
    parser = subparsers.add_parser("ros",
                                   description="Install, configure or update an existing ros distro.",  # this shows in the help for this command
                                   help="ros distro on an ubuntu machine",  # this shows in the parent parser
                                   formatter_class=argparse.ArgumentDefaultsHelpFormatter
                                   )
    common.add_ansible_arguments(parser)
    add_ros_arguments(parser)
    parser.set_defaults(func=parse_args)



#
# License: BSD
#    https://raw.githubusercontent.com/stonier/groot_ansible/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Python wrapped ansible scripts for installing and updating systems for
various use cases.
"""

##############################################################################
# Imports
##############################################################################

import argparse

from . import console
from . import ros
from . import update
from . import workstation

##############################################################################
# Constants
##############################################################################

__version__ = '0.2.2'

##############################################################################
# Main
##############################################################################


def version_string():
    return console.cyan + "Version" + console.reset + " : " + console.yellow + __version__ + console.reset


def main(args=None):
    """
    Entry point to the console 'groot-ansible' tool.
    """
    try:
        parser = argparse.ArgumentParser(
            description="version control index handling",
            epilog="And his noodly appendage reached forth to tickle the blessed...\n",
            formatter_class=argparse.RawDescriptionHelpFormatter)
        parser.add_argument('-v', '--version', action='version', version=version_string())
        subparsers = parser.add_subparsers(title='commands',
                                           help='valid commands for groot-ansible interactions')
        update.add_subparser(subparsers)
        ros.add_subparser(subparsers)
        workstation.add_subparser(subparsers)
        options = parser.parse_args(args)
        # options, unused_unknown_args = parser.parse_known_args(args)
        options.func(options)  # relay arg parsing to the subparser configured `set_defaults` function callback

    except KeyboardInterrupt:
        print('Interrupted by user!')




import os

from setuptools import find_packages
from setuptools import setup
from groot_ansible import __version__

# Setup installation dependencies, removing some so they
# can build on the ppa
install_requires = [
    'setuptools',
    'PyYAML',
    'ansible',
]


def roles():
    """
    Relative pathnames of all files in the groot_ansible/playbooks/roles folder
    """
    return [os.path.relpath(os.path.join(root, filename), 'groot_ansible')
            for root, _, filenames in os.walk('groot_ansible/playbooks/roles') for filename in filenames if '.git' not in root.split(os.sep)
            ]

setup(
    name='groot_ansible',
    version=__version__,
    packages=find_packages(exclude=['tests*', 'docs*']),
    package_data={
        'groot_ansible': [
            'playbooks/*.yaml',
        ] + roles(),
    },
    data_files=[],  # system files?
    install_requires=install_requires,
    author='Daniel Stonier',
    author_email='d.stonier@gmail.com',
    maintainer='Daniel Stonier',
    maintainer_email='d.stonier@gmail.com',
    url='http://github.com/stonier/groot_ansible',
    keywords=['catkin'],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities'
    ],
    description="system installation and update scripts",
    long_description="Python wrapped ansible for setting up and updating systems for various use cases.",
    license='BSD',
    # test_suite='tests',
    entry_points={
        'console_scripts': [
            'groot-ansible = groot_ansible:main',
        ],
    },
)
